searchData={"content_type":"text/markdown","items":[{"doc":"Adapter interface to the [Cowboy webserver](https://github.com/ninenines/cowboy).","ref":"Plug.Cowboy.html","title":"Plug.Cowboy","type":"module"},{"doc":"* `:net` - if using `:inet` (IPv4 only, the default) or `:inet6` (IPv6).\n\n  * `:ip` - the IP to bind the server to. Must be one of:\n\n      * a tuple in the format `{a, b, c, d}` with each value in `0..255` for IPv4,\n      * a tuple in the format `{a, b, c, d, e, f, g, h}` with each value in `0..65_535` for IPv6,\n      * or a tuple in the format `{:local, path}` for a Unix socket at the given `path`.\n\n    If you set an IPv6, the `:net` option will be automatically set to `:inet6`.\n    If both `:net` and `:ip` options are given, make sure they are compatible\n    (that is, give a IPv4 for `:inet` and IPv6 for `:inet6`).\n    Also, see the [*Loopback vs Public IP Addresses*\n    section](#module-loopback-vs-public-ip-addresses).\n\n  * `:port` - the port to run the server.\n    Defaults to `4000` (HTTP) and `4040` (HTTPS).\n    Must be `0` when `:ip` is a `{:local, path}` tuple.\n\n  * `:dispatch` - manually configure Cowboy's dispatch.\n    If this option is used, the given plug won't be initialized\n    nor dispatched to (and doing so becomes the user's responsibility).\n\n  * `:ref` - the reference name to be used.\n    Defaults to `plug.HTTP` (HTTP) and `plug.HTTPS` (HTTPS).\n    The default reference name does not contain the port, so in order\n    to serve the same plug on multiple ports you need to set the `:ref` accordingly.\n    For example, `ref: MyPlug_HTTP_4000`, `ref: MyPlug_HTTP_4001`, and so on.\n    This is the value that needs to be given on shutdown.\n\n  * `:compress` - if `true`, Cowboy will attempt to compress the response body.\n    Defaults to `false`.\n\n  * `:stream_handlers` - List of Cowboy `stream_handlers`,\n    see [Cowboy docs](https://ninenines.eu/docs/en/cowboy/2.12/manual/cowboy_http/).\n\n  * `:protocol_options` - Specifies remaining protocol options,\n    see the [Cowboy docs](https://ninenines.eu/docs/en/cowboy/2.12/manual/cowboy_http/).\n\n  * `:transport_options` - A keyword list specifying transport options,\n    see [Ranch docs](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch/).\n    By default `:num_acceptors` will be set to `100` and `:max_connections`\n    to `16_384`.\n\nAll other options given at the top level must configure the underlying\nsocket. For HTTP connections, those options are listed under\n[`ranch_tcp`](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch_tcp/).\nFor example, you can set `:ipv6_v6only` to true if you want to bind only\non IPv6 addresses.\n\nFor HTTPS (SSL) connections, those options are described in\n[`ranch_ssl`](https://ninenines.eu/docs/en/ranch/1.7/manual/ranch_ssl/).\nSee `https/3` for an example and read `Plug.SSL.configure/1` to\nunderstand about our SSL defaults.\n\nWhen using a Unix socket, OTP 21+ is required for `Plug.Static` and\n`Plug.Conn.send_file/3` to behave correctly.","ref":"Plug.Cowboy.html#module-options","title":"Options - Plug.Cowboy","type":"module"},{"doc":"Cowboy sets different limits on URL size, header length, number of\nheaders, and so on to protect your application from attacks. For example,\nthe request line length defaults to 10k, which means Cowboy will return\n`414` if a larger URL is given. You can change this under `:protocol_options`:\n\n    protocol_options: [max_request_line_length: 50_000]\n\nKeep in mind that increasing those limits can pose a security risk.\nOther times, browsers and proxies along the way may have equally strict\nlimits, which means the request will still fail or the URL will be\npruned. You can [consult all limits here](https://ninenines.eu/docs/en/cowboy/2.12/manual/cowboy_http/).","ref":"Plug.Cowboy.html#module-safety-limits","title":"Safety Limits - Plug.Cowboy","type":"module"},{"doc":"Should your application bind to a loopback address, such as `::1` (IPv6) or\n`127.0.0.1` (IPv4), or a public one, such as `::0` (IPv6) or `0.0.0.0`\n(IPv4)? It depends on how (and whether) you want it to be reachable from\nother machines.\n\nLoopback addresses are only reachable from the same host (`localhost` is\nusually configured to resolve to a loopback address). You may wish to use one if:\n\n  * Your app is running in a development environment (such as your laptop) and\n    you don't want others on the same network to access it.\n  * Your app is running in production, but behind a reverse proxy. For\n    example, you might have [nginx](https://nginx.org/en/) bound to a public\n    address and serving HTTPS, but forwarding the traffic to your application\n    running on the same host. In that case, having your app bind to the\n    loopback address means that nginx can reach it, but outside traffic can\n    only reach it via nginx.\n\nPublic addresses are reachable from other hosts. You may wish to use one if:\n\n  * Your app is running in a container. In this case, its loopback address is\n    reachable only from within the container; to be accessible from outside the\n    container, it needs to bind to a public IP address.\n  * Your app is running in production without a reverse proxy, using Cowboy's\n    SSL support.","ref":"Plug.Cowboy.html#module-loopback-vs-public-ip-addresses","title":"Loopback vs Public IP Addresses - Plug.Cowboy","type":"module"},{"doc":"You can configure which exceptions are logged via `:log_exceptions_with_status_code`\napplication environment variable. If the status code returned by `Plug.Exception.status/1`\nfor the exception falls into any of the configured ranges, the exception is logged.\nBy default it's set to `[500..599]`.\n\n    config :plug_cowboy,\n      log_exceptions_with_status_code: [400..599]\n\nBy default, `Plug.Cowboy` includes the entire `conn` to the log metadata for exceptions.\nHowever, this metadata may contain sensitive information such as security headers or\ncookies, which may be logged in plain text by certain logging backends. To prevent this,\nyou can configure the `:conn_in_exception_metadata` option to not include the `conn` in the metadata.\n\n    config :plug_cowboy,\n      conn_in_exception_metadata: false","ref":"Plug.Cowboy.html#module-logging","title":"Logging - Plug.Cowboy","type":"module"},{"doc":"`Plug.Cowboy` uses the [`telemetry` library](https://github.com/beam-telemetry/telemetry)\nfor instrumentation. The following span events are published during each request:\n\n  * `[:cowboy, :request, :start]` - dispatched at the beginning of the request\n  * `[:cowboy, :request, :stop]` - dispatched at the end of the request\n  * `[:cowboy, :request, :exception]` - dispatched at the end of a request that exits\n\nA single event is published when the request ends with an early error:\n  * `[:cowboy, :request, :early_error]` - dispatched for requests terminated early by Cowboy\n\nSee [`cowboy_telemetry`](https://github.com/beam-telemetry/cowboy_telemetry#telemetry-events)\nfor more details on the events and their measurements and metadata.\n\nTo opt-out of this default instrumentation, you can manually configure\nCowboy with the option:\n\n    stream_handlers: [:cowboy_stream_h]","ref":"Plug.Cowboy.html#module-instrumentation","title":"Instrumentation - Plug.Cowboy","type":"module"},{"doc":"`Plug.Cowboy` supports upgrading HTTP requests to WebSocket connections via\nthe use of the `Plug.Conn.upgrade_adapter/3` function, called with `:websocket` as the second\nargument. Applications should validate that the connection represents a valid WebSocket request\nbefore calling this function (Cowboy will validate the connection as part of the upgrade\nprocess, but does not provide any capacity for an application to be notified if the upgrade is\nnot successful). If an application wishes to negotiate WebSocket subprotocols or otherwise set\nany response headers, it should do so before calling `Plug.Conn.upgrade_adapter/3`.\n\nThe third argument to `Plug.Conn.upgrade_adapter/3` defines the details of how Plug.Cowboy\nshould handle the WebSocket connection, and must take the form `{handler, handler_opts,\nconnection_opts}`, where values are as follows:\n\n* `handler` is a module which implements the\n  [`:cowboy_websocket`](https://ninenines.eu/docs/en/cowboy/2.6/manual/cowboy_websocket/)\n  behaviour. Note that this module will NOT have its `c:cowboy_websocket.init/2` callback\n  called; only the 'later' parts of the `:cowboy_websocket` lifecycle are supported\n* `handler_opts` is an arbitrary term which will be passed as the argument to\n  `c:cowboy_websocket.websocket_init/1`\n* `connection_opts` is a map with any of [Cowboy's websockets options](https://ninenines.eu/docs/en/cowboy/2.6/manual/cowboy_websocket/#_opts)","ref":"Plug.Cowboy.html#module-websocket-support","title":"WebSocket support - Plug.Cowboy","type":"module"},{"doc":"Returns a supervisor child spec to start Cowboy under a supervisor.\n\nIt supports all options as specified in the module documentation plus it\nrequires the following two options:\n\n  * `:scheme` - either `:http` or `:https`\n  * `:plug` - such as `MyPlug` or `{MyPlug, plug_opts}`","ref":"Plug.Cowboy.html#child_spec/1","title":"Plug.Cowboy.child_spec/1","type":"function"},{"doc":"Assuming your Plug module is named `MyApp` you can add it to your\nsupervision tree by using this function:\n\n    children = [\n      {Plug.Cowboy, scheme: :http, plug: MyApp, options: [port: 4040]}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Plug.Cowboy.html#child_spec/1-examples","title":"Examples - Plug.Cowboy.child_spec/1","type":"function"},{"doc":"Runs cowboy under HTTP.","ref":"Plug.Cowboy.html#http/3","title":"Plug.Cowboy.http/3","type":"function"},{"doc":"# Starts a new interface:\n    Plug.Cowboy.http(MyPlug, [], port: 80)\n\n    # The interface above can be shut down with:\n    Plug.Cowboy.shutdown(MyPlug.HTTP)","ref":"Plug.Cowboy.html#http/3-example","title":"Example - Plug.Cowboy.http/3","type":"function"},{"doc":"Runs cowboy under HTTPS.\n\nBesides the options described in the module documentation,\nthis function sets defaults and accepts all options defined\nin `Plug.SSL.configure/1`.","ref":"Plug.Cowboy.html#https/3","title":"Plug.Cowboy.https/3","type":"function"},{"doc":"# Starts a new interface:\n    Plug.Cowboy.https(\n      MyPlug,\n      [],\n      port: 443,\n      password: \"SECRET\",\n      otp_app: :my_app,\n      keyfile: \"priv/ssl/key.pem\",\n      certfile: \"priv/ssl/cert.pem\",\n      dhfile: \"priv/ssl/dhparam.pem\"\n    )\n\n    # The interface above can be shut down with:\n    Plug.Cowboy.shutdown(MyPlug.HTTPS)","ref":"Plug.Cowboy.html#https/3-example","title":"Example - Plug.Cowboy.https/3","type":"function"},{"doc":"Shutdowns the given reference.","ref":"Plug.Cowboy.html#shutdown/1","title":"Plug.Cowboy.shutdown/1","type":"function"},{"doc":"Process to drain cowboy connections at shutdown.\n\nWhen starting `Plug.Cowboy` in a supervision tree, it will create a listener that receives\nrequests and creates a connection process to handle that request. During shutdown, a\n`Plug.Cowboy` process will immediately exit, closing the listener and any open connections\nthat are still being served. However, in most cases, it is desirable to allow connections\nto complete before shutting down.\n\nThis module provides a process that during shutdown will close listeners and wait\nfor connections to complete. It should be placed after other supervised processes that\nhandle cowboy connections.","ref":"Plug.Cowboy.Drainer.html","title":"Plug.Cowboy.Drainer","type":"module"},{"doc":"The following options can be given to the child spec:\n\n  * `:refs` - A list of refs to drain. `:all` is also supported and will drain all cowboy\n    listeners, including those started by means other than `Plug.Cowboy`.\n\n  * `:id` - The ID for the process.\n    Defaults to `Plug.Cowboy.Drainer`.\n\n  * `:shutdown` - How long to wait for connections to drain.\n    Defaults to 5000ms.\n\n  * `:check_interval` - How frequently to check if a listener's\n    connections have been drained. Defaults to 1000ms.","ref":"Plug.Cowboy.Drainer.html#module-options","title":"Options - Plug.Cowboy.Drainer","type":"module"},{"doc":"# In your application\n    def start(_type, _args) do\n      children = [\n        {Plug.Cowboy, scheme: :http, plug: MyApp, options: [port: 4040]},\n        {Plug.Cowboy, scheme: :https, plug: MyApp, options: [port: 4041]},\n        {Plug.Cowboy.Drainer, refs: [MyApp.HTTP, MyApp.HTTPS]}\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","ref":"Plug.Cowboy.Drainer.html#module-examples","title":"Examples - Plug.Cowboy.Drainer","type":"module"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-7-3","title":"v2.7.3 - Changelog","type":"extras"},{"doc":"* Ensure errors from Cowboy 2.13 are correctly translated","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-7-2","title":"v2.7.2 - Changelog","type":"extras"},{"doc":"* Ensure `crash_reason` in metadata is always a tuple","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-7-1","title":"v2.7.1 - Changelog","type":"extras"},{"doc":"* Support Cowboy 2.11","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-7-0","title":"v2.7.0 - Changelog","type":"extras"},{"doc":"* Do not allow Cowboy 2.11 due to backwards incompatible changes","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-6-2","title":"v2.6.2 - Changelog","type":"extras"},{"doc":"* Fix warnings on Elixir v1.15+","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-6-1","title":"v2.6.1 - Changelog","type":"extras"},{"doc":"* Allow for opt-out of conn metadata on exception logs\n * Support `:check_interval` in drainer (in addition to `:drain_check_interval`)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-6-0","title":"v2.6.0 - Changelog","type":"extras"},{"doc":"* Support websocket upgrades\n  * Require Plug v1.14+ and Elixir v1.10+","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-5-2","title":"v2.5.2 - Changelog","type":"extras"},{"doc":"* Fix warnings when running on telemetry 1.x","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-5-1","title":"v2.5.1 - Changelog","type":"extras"},{"doc":"* Allow to configure which errors should be logged\n  * Support telemetry 0.4.x or 1.x","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-5-0","title":"v2.5.0 - Changelog","type":"extras"},{"doc":"* Return `:conn` as Logger metadata on translator\n  * Support Ranch 2.0\n  * Support the `:net` option so developers can work with keyword lists\n  * Remove previously deprecated options","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-4-1-2020-10-31","title":"v2.4.1 (2020-10-31) - Changelog","type":"extras"},{"doc":"* Properly format linked exits","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-4-0-2020-10-11","title":"v2.4.0 (2020-10-11) - Changelog","type":"extras"},{"doc":"* Add [cowboy_telemetry](https://github.com/beam-telemetry/cowboy_telemetry/) as a dependency and enable it by default","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"Plug.Cowboy requires Elixir v1.7 or later.","ref":"changelog.html#v2-3-0-2020-06-11","title":"v2.3.0 (2020-06-11) - Changelog","type":"extras"},{"doc":"* The telemetry events added in version v2.2.0 does not work as expected. The whole v2.2.x branch has been retired in favor of v2.3.0.","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-2-2-2020-05-25","title":"v2.2.2 (2020-05-25) - Changelog","type":"extras"},{"doc":"* Emit telemetry event for Cowboy early errors\n  * Improve error messages for Cowboy early errors","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-2-1-2020-04-21","title":"v2.2.1 (2020-04-21) - Changelog","type":"extras"},{"doc":"* Use proper telemetry metadata for exceptions","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-2-0-2020-04-21","title":"v2.2.0 (2020-04-21) - Changelog","type":"extras"},{"doc":"* Include telemetry support","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-1-3-2020-04-14","title":"v2.1.3 (2020-04-14) - Changelog","type":"extras"},{"doc":"* Properly support the :options option before removal","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-1-2-2020-01-28","title":"v2.1.2 (2020-01-28) - Changelog","type":"extras"},{"doc":"* Properly deprecate the :timeout option before removal","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-1-1-2020-01-08","title":"v2.1.1 (2020-01-08) - Changelog","type":"extras"},{"doc":"* Improve docs and simplify child spec API","ref":"changelog.html#enhancement","title":"Enhancement - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-1-0-2019-06-27","title":"v2.1.0 (2019-06-27) - Changelog","type":"extras"},{"doc":"* Add `Plug.Cowboy.Drainer` for connection draining","ref":"changelog.html#enhancement","title":"Enhancement - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-0-2-2019-03-18","title":"v2.0.2 (2019-03-18) - Changelog","type":"extras"},{"doc":"* Unwrap `Plug.Conn.WrapperError` on handler error\n  * Include `crash_reason` as logger metadata","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#v2-0-1-2018-12-13","title":"v2.0.1 (2018-12-13) - Changelog","type":"extras"},{"doc":"* Respect `:read_length` and `:read_timeout` in `read_body` with Cowboy 2","ref":"changelog.html#bug-fixes","title":"Bug fixes - Changelog","type":"extras"},{"doc":"Extract `Plug.Adapters.Cowboy2` from Plug into `Plug.Cowboy`","ref":"changelog.html#v2-0-0-2018-10-20","title":"v2.0.0 (2018-10-20) - Changelog","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}